@@ -6,7 +6,6 @@
 ################################################################################
 from __future__ import absolute_import, division, with_statement
 
-import sys
 from datetime import timedelta
 
 from past.utils import old_div
@@ -28,6 +27,7 @@ from ..openrdf.util.strings import to_native_string
 
 from six import python_2_unicode_compatible
 
+
 def _split_proxy(proxy):
     """
     Split a proxy definition of the form PROTOCOL://HOST:PORT into components.
@@ -138,7 +138,7 @@ class Catalog(Service):
         return self._instance_from_url(Repository, to_native_string(self.url) + "/repositories/" + quote(name))
 
 
-class Client(Catalog):
+class Client(Service):
     def getVersion(self):
         return jsonRequest(self, "GET", "/version")
 
@@ -219,7 +219,6 @@ class Client(Catalog):
     def listUserEffectiveAccess(self, name):
         return jsonRequest(self, "GET", "/users/" + quote(name) + "/effectiveAccess")
 
-
     def listUserPermissions(self, name):
         return jsonRequest(self, "GET", "/users/" + quote(name) + "/permissions")
 
@@ -292,6 +291,31 @@ class Client(Catalog):
         nullRequest(self, "DELETE", "/roles/" + quote(role) + "/security-filters/" + to_native_string(_type) + "?" +
             urlenc(s=s, p=p, o=o, g=g))
 
+    def getUserData(self, key):
+        try:
+            return jsonRequest(self, "GET",
+                               "/users/%s/data/%s" % (quote(self.user, safe=''),
+                                                      quote(key, safe='')),
+                               )
+        except RequestError as e:
+            if e.status == 404:
+                return None
+            raise
+
+    def setUserData(self, key, data):
+        nullRequest(self, "PUT", "/users/%s/data/%s" % (quote(self.user, safe=''),
+                                                        quote(key, safe='')),
+                    body=data)
+
+    def deleteUserData(self, key):
+        nullRequest(self, "DELETE", "/users/%s/data/%s" % (quote(self.user, safe=''),
+                                                           quote(key, safe='')))
+
+    def listUserData(self):
+        entries = jsonRequest(self, "GET",
+                              "/users/%s/data/" % quote(self.user, safe=''))
+        return [entry['id'] for entry in entries]
+
 
 class Repository(Service):
     sessionAlive = None
@@ -423,18 +447,29 @@ class Repository(Service):
                     encode_json(quads), content_type="application/json")
 
     def loadData(self, data, rdf_format, base_uri=None, context=None,
-                 commit_every=None, content_encoding=None, attributes=None):
+                 commit_every=None, content_encoding=None, attributes=None,
+                 json_ld_store_source=None,
+                 json_ld_context=None, allow_external_references=None,
+                 external_reference_timeout=None):
         nullRequest(self, "POST",
-                    "/statements?" + urlenc(context=context,
-                                            baseURI=base_uri,
-                                            commit=commit_every,
-                                            attributes=attributes and encode_json(attributes)),
+                    "/statements?" + urlenc(
+                        context=context,
+                        baseURI=base_uri,
+                        commit=commit_every,
+                        attributes=attributes and encode_json(attributes),
+                        jsonLdStoreSource=json_ld_store_source,
+                        jsonLdContext=json_ld_context and fix_json_ld_context(json_ld_context),
+                        externalReferences=allow_external_references,
+                        extermalReferenceTimeout=external_reference_timeout),
                     data,
                     content_type=Format.mime_type_for_format(rdf_format),
-                    content_encoding=content_encoding)
+                    content_encoding=content_encoding,)
 
     def loadFile(self, file, rdf_format, baseURI=None, context=None, serverSide=False,
-                 commitEvery=None, content_encoding=None, attributes=None):
+                 commitEvery=None, content_encoding=None, attributes=None,
+                 json_ld_store_source=None,
+                 json_ld_context=None, allow_external_references=None,
+                 external_reference_timeout=None):
         mime = Format.mime_type_for_format(rdf_format)
 
         if serverSide:
@@ -452,7 +487,11 @@ class Repository(Service):
 
         with body_context as body:
             params = urlenc(file=file, context=context, baseURI=baseURI, commit=commitEvery,
-                            attributes=attributes and encode_json(attributes))
+                            attributes=attributes and encode_json(attributes),
+                            jsonLdStoreSource=json_ld_store_source,
+                            jsonLdContext=json_ld_context and fix_json_ld_context(json_ld_context),
+                            externalReferences=allow_external_references,
+                            extermalReferenceTimeout=external_reference_timeout)
             nullRequest(self, "POST", "/statements?" + params, body,
                         content_type=mime, content_encoding=content_encoding)
 
@@ -466,12 +505,11 @@ class Repository(Service):
     def deleteStatementsById(self, ids):
         nullRequest(self, "POST", "/statements/delete?ids=true", encode_json(ids), content_type="application/json")
 
-    def evalFreeTextSearch(self, pattern, index=None, infer=False, callback=None, limit=None, offset=None):
+    def evalFreeTextSearch(self, pattern, index=None, infer=False, limit=None, offset=None):
         """Use free-text indices to search for the given pattern.
         Returns an array of statements."""
         return jsonRequest(self, "GET", "/freetext",
-                           urlenc(pattern=pattern, infer=infer, limit=limit, offset=offset, index=index),
-                           callback=callback)
+                           urlenc(pattern=pattern, infer=infer, limit=limit, offset=offset, index=index))
 
     def listFreeTextIndices(self):
         """List the names of free-text indices defined in this
@@ -708,6 +746,11 @@ class Repository(Service):
     def enableTripleCache(self, size=None):
         nullRequest(self, "PUT", "/tripleCache?" + urlenc(size=size))
 
+    def warmup(self, includeStrings, includeTriples, indices):
+        nullRequest(self, "PUT", "/warmup?" + urlenc(includeStrings=includeStrings,
+                                                     includeTriples=includeTriples,
+                                                     index=indices))
+           
     def openSession(self, autocommit=False, lifetime=None, loadinitfile=False):
         if self.sessionAlive:
             # A session is already active.  Do nothing.
@@ -969,3 +1012,14 @@ def time_in_seconds(t):
         # Can't use total_seconds() in Python 2.6
         return t.seconds + t.days * 24 * 3600
     return int(t)
+
+
+def fix_json_ld_context(context):
+    """
+    Take a dict or a string and return a JSON string.
+
+    If the input dict does not contain "@context", wrap it.
+    """
+    if not isinstance(context, dict) or "@context" not in context:
+        context = {"@context": context}
+    return encode_json(context)
